﻿
using Ef6TestApp;
using Microsoft.EntityFrameworkCore;
using static System.Console;

WriteLine("Seeding data...");

var db = new ShopContext();
await db.Database.EnsureDeletedAsync();
await db.Database.EnsureCreatedAsync();


var prod = new Product { Name = "WallBox", Price = 999.9m };
var order = new Order { Customer = "Thomass", Product = prod, Quantity = 10 };
db.Add(order);
await db.SaveChangesAsync();
var save1 = DateTime.UtcNow;

Thread.Sleep(100);
prod.Price -= 100;
order.Quantity = 11;
order.Customer = "Tchomas";
await db.SaveChangesAsync();
var save2 = DateTime.UtcNow;


Thread.Sleep(100);
prod.Name = "WallBox T1000";
prod.Price -= 100;
order.Quantity = 15;
order.Customer = "Thomas";
await db.SaveChangesAsync();
var save3 = DateTime.UtcNow;


QueryAndDisplay(save1);
QueryAndDisplay(save2);
QueryAndDisplay(save3);


WriteLine("Hit enter to terminate...");
ReadLine();



void QueryAndDisplay(DateTime pointInTime)
{
    var orderHistory = db.Orders!
    .TemporalAsOf(pointInTime)
    .Include(x => x.Product)
    .Where(x => x.Id == order!.Id)
    .Select(x => new
    {
        Order = x,
        PeriodStart = EF.Property<DateTime>(x, "PeriodStart"),
        PeriodEnd = EF.Property<DateTime>(x, "PeriodEnd"),
    }).Single();

    WriteLine($"Order valid from: {orderHistory.PeriodStart:O}");
    WriteLine($"              to: {orderHistory.PeriodEnd:O}");
    WriteLine($"        Customer: {orderHistory.Order.Customer}");
    WriteLine($"        Quantity: {orderHistory.Order.Quantity}");
    WriteLine($"    Product Name: {orderHistory.Order.Product?.Name}");
    WriteLine($"   Product Price: {orderHistory.Order.Product?.Price:C}");
    WriteLine("");
}
